@@ -67,6 +67,10 @@ class SphericalTensor:
     def shape(self):
         return self.ten.shape
 
+    @property
+    def device(self):
+        return self.ten.device
+
     def self_like(self, new_data_ten: torch.Tensor):
         """
         Args:
@@ -157,7 +161,7 @@ class SphericalTensor:
         elif len(self.rep_dims) == 2:
             dimid_kept = 1 - dotdim_idx
             assert other.ten.shape[self.rep_dims[dimid_kept]] == 1
-            return SphericalTensor(
+            return self.__class__(
                 out_ten,
                 rep_dims=(self.rep_dims[dimid_kept],),
                 metadata=self.metadata[dimid_kept].unsqueeze(0),
@@ -199,7 +203,7 @@ class SphericalTensor:
         elif len(self.rep_dims) == 2:
             # A little trick here
             dimid_kept = 1 - dotdim_idx
-            return SphericalTensor(
+            return self.__class__(
                 out_ten,
                 rep_dims=(self.rep_dims[dimid_kept],),
                 metadata=self.metadata[dimid_kept].unsqueeze(0),
@@ -227,7 +231,7 @@ class SphericalTensor:
             self.rep_layout[0],
             other.rep_layout[0],
         )
-        return SphericalTensor(
+        return self.__class__(
             out_ten,
             rep_dims=(odim, odim + 1),
             metadata=out_metadata,
@@ -241,7 +245,7 @@ class SphericalTensor:
         """
         assert len(self.rep_dims) == 1
         assert torch.all(torch.fmod(self.metadata[0], stride) == 0), (
-            f"The number of channels for theSphericalTensor to be folded must be multiples of "
+            f"The number of channels for the SphericalTensor to be folded must be multiples of "
             f"stride, got ({self.metadata}, {stride}) instead"
         )
         new_ten = self.ten.unflatten(
@@ -258,7 +262,36 @@ class SphericalTensor:
             self.num_channels = new_num_channels
             return self
         else:
-            return SphericalTensor(
+            return self.__class__(
+                new_ten,
+                rep_dims=self.rep_dims,
+                metadata=new_metadata,
+                rep_layout=new_rep_layout,
+                num_channels=new_num_channels,
+            )
+
+    def unfold(self, update_self=False) -> "SphericalTensor":
+        """
+        Contract one trailing dimension into the representation dimension.
+        """
+        assert len(self.rep_dims) == 1
+        assert self.ten.dim() > self.rep_dims[0], "No trailing dimension to unfold"
+        stride = self.ten.shape[self.rep_dims[0] + 1]
+        new_ten = self.ten.flatten(
+            self.rep_dims[0],
+            self.rep_dims[0] + 1,
+        )
+        new_metadata = self.metadata * stride
+        new_rep_layout = (self.rep_layout[0].repeat_interleave(stride, dim=1),)
+        new_num_channels = (self.num_channels[0] * stride,)
+        if update_self:
+            self.ten = new_ten
+            self.metadata = new_metadata
+            self.rep_layout = new_rep_layout
+            self.num_channels = new_num_channels
+            return self
+        else:
+            return self.__class__(
                 new_ten,
                 rep_dims=self.rep_dims,
                 metadata=new_metadata,
@@ -283,7 +316,7 @@ class SphericalTensor:
             self.num_channels = num_channels_t
             return self
         else:
-            return SphericalTensor(
+            return self.__class__(
                 ten_t,
                 rep_dims=dims_t,
                 metadata=metadata_t,
@@ -387,3 +420,130 @@ class SphericalTensor:
         self.ten = self.ten.to(device)
         self.rep_layout = tuple(layout.to(device) for layout in self.rep_layout)
         return self
+
+
+@dataclass
+class O3Tensor(SphericalTensor):
+    """
+    The O3Tensor class tracks the O(3) representation indices with additional
+    parity (p) index tracker.
+
+    Two indexing tensors are maintained, ``metadata`` and ``rep_layout``.
+
+    Attributes:
+        data_ten (torch.Tensor): The tensor storing auto-differentiable data.
+        metadata (torch.LongTensor): The minimal information which specifies
+            the number of non-degenerate feature channels for each dimension
+            and each l and p (and the associated m(s)).
+        rep_layout (Tuple[torch.LongTensor]): Stores the pointer (l[i], m[i], n[i], p[i])
+            for each index i along the SO(3) representation dimension of the
+            flattened data tensor.
+
+    Args:
+        data_ten (torch.Tensor): the underlying flattened data tensor in the
+             strictly incremental [l,m,-p,n] order, for which the rep_dim has N elements
+        rep_dims (tuple of ints): the spherical tensor representation dimension(s). When
+             n_rep_dims > 1, must be contiguous dimensions.
+        metadata (torch.LongTensor): (n_rep_dims x (2 * n_l)) specification of the number of unique
+             channels (n) for each angular momentum index l.
+        rep_layout (tuple of ints, optional): the (4xN) index tensor for (l[i],m[i],n[i],p[i]).
+        num_channels (tuple of ints, optional): the number of unique channels per rep_dim.
+    """
+
+    def __init__(
+        self,
+        data_ten: torch.Tensor,
+        rep_dims: Tuple[int, ...],
+        metadata: torch.LongTensor,
+        rep_layout: Optional[Tuple[torch.LongTensor, ...]] = None,
+        num_channels: Optional[Tuple[int, ...]] = None,
+    ):
+        self.ten = data_ten
+        self.metadata = metadata
+        assert self.metadata.shape[1] % 2 == 0
+        self.rep_dims = rep_dims
+        assert self.rep_dims[-1] < self.ten.dim()
+        self._norm_eps = 1e-4
+        if rep_layout:
+            self.rep_layout = rep_layout
+        else:
+            if len(rep_dims) > 1:
+                assert rep_dims[1] - rep_dims[0] == 1
+            self.rep_layout = self.generate_rep_layout()
+        if num_channels:
+            self.num_channels = num_channels
+        else:
+            self.num_channels = tuple(torch.sum(self.metadata, dim=1).long().tolist())
+
+    def self_like(self, new_data_ten: torch.Tensor):
+        """
+        Args:
+            new_data_ten: The underlying data of the new O3Tensor.
+
+        Returns:
+             A O3Tensor with identical layout but new data tensor.
+        """
+        return O3Tensor(
+            new_data_ten,
+            rep_dims=self.rep_dims,
+            metadata=self.metadata,
+            rep_layout=self.rep_layout,
+            num_channels=self.num_channels,
+        )
+
+    @staticmethod
+    def generate_rep_layout_1d_(metadata1d) -> torch.LongTensor:
+        num_distinct_ls = metadata1d.size(0) // 2
+        n_irreps_per_l = torch.arange(start=0, end=num_distinct_ls) * 2 + 1
+        n_irreps_per_lp = n_irreps_per_l.repeat_interleave(2)
+        ls_metadata = torch.arange(start=0, end=num_distinct_ls).repeat_interleave(2)
+        end_channelids = torch.cumsum(metadata1d, dim=0)
+        start_channelids = torch.cat([torch.LongTensor([0]), end_channelids[:-1]])
+        dst_ls = torch.repeat_interleave(ls_metadata, n_irreps_per_lp * metadata1d)
+        dst_ms = torch.repeat_interleave(
+            torch.cat([torch.arange(-l, l + 1) for l in ls_metadata]),
+            torch.repeat_interleave(metadata1d, n_irreps_per_lp),
+        )
+        ns = torch.arange(metadata1d.sum())
+        dst_ns = torch.cat(
+            [
+                ns[start_channelids[l] : end_channelids[l]].repeat(n_irreps)
+                for l, n_irreps in enumerate(n_irreps_per_lp)
+            ]
+        )
+        ps = torch.LongTensor([1, -1]).repeat(num_distinct_ls)
+        dst_ps = torch.repeat_interleave(ps, n_irreps_per_lp * metadata1d)
+        rep_layout = torch.stack([dst_ls, dst_ms, dst_ns, dst_ps], dim=0).long()
+        assert rep_layout.shape[1] == torch.sum(n_irreps_per_lp * metadata1d)
+
+        return rep_layout
+
+    @classmethod
+    def from_so3(cls, so3_ten: SphericalTensor, parity=1):
+        metadata = so3_ten.metadata
+        if parity == 1:
+            o3_metadata = torch.stack(
+                [metadata, torch.zeros_like(metadata)], dim=2
+            ).view(metadata.shape[0], -1)
+            o3_layout = tuple(
+                torch.cat([layout, torch.ones(1, layout.shape[1])], dim=0)
+                for layout in so3_ten.rep_layout
+            )
+        elif parity == -1:
+            o3_metadata = torch.stack(
+                [torch.zeros_like(metadata), metadata], dim=2
+            ).view(metadata.shape[0], -1)
+            o3_layout = tuple(
+                torch.cat([layout, torch.ones(1, layout.shape[1]).neg()], dim=0)
+                for layout in so3_ten.rep_layout
+            )
+        else:
+            raise NotImplementedError
+
+        return O3Tensor(
+            so3_ten.ten,
+            rep_dims=so3_ten.rep_dims,
+            metadata=o3_metadata,
+            rep_layout=o3_layout,
+            num_channels=so3_ten.num_channels,
+        )
